package salaComunitaria.entities;

import java.util.Date;
import java.util.Set;

import javax.persistence.*;

import org.hibernate.annotations.ResultCheckStyle;
import org.hibernate.annotations.SQLDelete;
import org.hibernate.annotations.Where;

@Entity
@SQLDelete(sql = "UPDATE BATCHES SET state = 'DELETED' WHERE BATCH_ID = ?", check = ResultCheckStyle.COUNT)
@Where(clause = "state <> 'DELETED'")
@NamedQuery(name = "batches.FindById", query = "SELECT a FROM Batch a WHERE a.Id = :id")
@Table(name="BATCHES")
public class Batch {
	
	//Attributes
	@Id @GeneratedValue
	@Column(name="BATCH_ID")
	private long Id;
	
	@Column(nullable=false)
	private String name;
	
	@Column(nullable=false)
	private String category;
	
	@Column(nullable=false)
	private String unit;
	
	@Column(nullable=false)
	private int quantity;
	
	@Column(nullable=false)
	private double price;
	
	@Column(nullable=false)
	private Date expiration;
	
	@Column(nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date creationDate;
	
	@Column(nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date lastModificationDate;
	
	@Column(nullable=false)
	private String description;
	
	@Column(nullable=false)
	private String batchIngredientsQuantity;

	@Column(nullable=false)
	private String batchPackagingElementsQuantity;

	
	//Los estados que se van a manejar son ACTIVE y DELETED
    @Column(nullable=false, name="state")
    private String state = "ACTIVE";
	
	//Relations
	@ManyToMany
	@JoinTable(	name="BATCH_INGREDIENTS",
				joinColumns=@JoinColumn(name="B_ID",referencedColumnName="BATCH_ID"),
				inverseJoinColumns=@JoinColumn(name="RM_ID",referencedColumnName="RAW_MATERIAL_ID")
	)
	private Set<RawMaterial> batchIngredients;
	

	@ManyToMany(cascade = {
            CascadeType.PERSIST,
            CascadeType.MERGE
    })
	@JoinTable(	name="PACKAGING_ELEMENTS",
				joinColumns=@JoinColumn(name="B_ID",referencedColumnName="BATCH_ID"),
				inverseJoinColumns=@JoinColumn(name="I_ID",referencedColumnName="INPUT_ID")
	)
	private Set<Input> batchPackagingElements;
	
	public Batch() {
		super();
	}

	@PrePersist
	protected void onCreate() {
		creationDate = new Date(); // Fecha actual cuando se crea la entidad
		lastModificationDate = new Date(); // Fecha actual cuando se crea la entidad
	}

	@PreUpdate
	protected void onUpdate() {
		lastModificationDate = new Date();
	}
	
	//Getters and Setters
	
	public long getId() {
		return Id;
	}

	public void setId(long id) {
		Id = id;
	}
	
	public double getPrice() {
		return price;
	}

	public void setPrice(double price) {
		this.price = price;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}

	public String getUnit() {
		return unit;
	}

	public void setUnit(String unit) {
		this.unit = unit;
	}

	public int getQuantity() {
		return quantity;
	}

	public void setQuantity(int quantity) {
		this.quantity = quantity;
	}


	public Date getExpiration() {
		return expiration;
	}

	public void setExpiration(Date expiration) {
		this.expiration = expiration;
	}

	public Date getCreationDate() {
		return creationDate;
	}

	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}

	public Date getLastModificationDate() {
		return lastModificationDate;
	}

	public void setLastModificationDate(Date lastModificationDate) {
		this.lastModificationDate = lastModificationDate;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Set<RawMaterial> getBatchIngredients() {
		return batchIngredients;
	}

	public void setBatchIngredients(Set<RawMaterial> batchIngredients) {
		this.batchIngredients = batchIngredients;
	}

	public Set<Input> getBatchPackagingElements() {
		return batchPackagingElements;
	}

	public void setBatchPackagingElements(Set<Input> batchPackagingElements) {
		this.batchPackagingElements = batchPackagingElements;
	}
	

	public String getBatchIngredientsQuantity() {
		return batchIngredientsQuantity;
	}

	public void setBatchIngredientsQuantity(String batchIngredientsQuantity) {
		this.batchIngredientsQuantity = batchIngredientsQuantity;
	}

	public String getBatchPackagingElementsQuantity() {
		return batchPackagingElementsQuantity;
	}

	public void setBatchPackagingElementsQuantity(String batchPackagingElementsQuantity) {
		this.batchPackagingElementsQuantity = batchPackagingElementsQuantity;
	}

	@Override
	public String toString() {
		return "Batch [Id=" + Id + ", name=" + name + ", category=" + category + ", unit=" + unit + ", quantity="
				+ quantity + ", expirationDate=" + expiration + ", creationDate=" + creationDate
				+ ", lastModificationDate=" + lastModificationDate + ", description=" + description + "]";
	}
	
	
}
