package salaComunitaria.entities;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

@Entity
@Table(name="EVENTS")
@Inheritance(strategy=InheritanceType.JOINED)
@DiscriminatorColumn(name="EVENT_TYPE")
@JsonTypeInfo(
		  use = JsonTypeInfo.Id.NAME, 
		  include = JsonTypeInfo.As.PROPERTY, 
		  property = "type")
		@JsonSubTypes({
		    @JsonSubTypes.Type(value = InventoryEvent.class, name = "inventory"),
		    @JsonSubTypes.Type(value = SaleEvent.class, name = "sales")
		})
public abstract class Event {

	@Id @GeneratedValue
	@Column(name="EVENT_ID")
	private long id;
	
	@Column(nullable=false)
	private String title;
	
	@Column(nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date envenDate;
	
	@Column(nullable=false)
	private String description;

	@ManyToOne(optional = true)
    @JoinColumn(name = "USER_ID")
    private User user;

	
	public Event() {
		super();
	}
	
	@PrePersist
	protected void onCreate() {
		envenDate = new Date(); // Fecha actual cuando se crea la entidad
		
	}

	//Getters and Setters
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public Date getEnvenDate() {
		return envenDate;
	}

	public void setEnvenDate(Date envenDate) {
		this.envenDate = envenDate;
	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}
	
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	@Override
	public String toString() {
		return "Event [id=" + id + ", title=" + title + ", envenDate=" + envenDate + ", description=" + description
				+ "]";
	}
	
	
}
